#include <_Custom_Operacional/Estrategia/Strategy.mqh>
#include <_Custom_Charts/Indicadores/NsVPR.mqh>

class NsVPRStg : public Strategy{
private:
    //Ponteiro do indicador da strategy
    NsVPR* nsvprIndicador;
    //Relação com os buffers
    ENUM_MARKET_TO_BUFFER_RELATION relatMxM;
    ENUM_MARKET_TO_BUFFER_RELATION relatMxS;
    ENUM_MARKET_TO_BUFFER_RELATION relatMxD;
    ENUM_MARKET_TO_BUFFER_RELATION relatMeio;
    ENUM_MARKET_TO_BUFFER_RELATION relatMiD;
    ENUM_MARKET_TO_BUFFER_RELATION relatMiS;
    ENUM_MARKET_TO_BUFFER_RELATION relatMiM;

    int totalIndicacoesCompra;
    int totalIndicacoesVenda;

public:
    NsVPRStg(){};
    ~NsVPRStg(){};

    //Passando valores do NsVPR
    void set(double variacaoPercentil_, int atrasoVelas_);
    //Método genérico a ser implementado pelas classes filhas da definição do indicador da strategy
    virtual void set();
    //Define a relação do mercado com os buffers do indicador
    virtual void setBuffersRelation();
    //Método genérico a ser implementado pelas classes filhas para retorno da indicação atual da strategy
    virtual ENUM_STRATEGY_SIGN getSign();
    //Método genérico para setar o valor de relação dos buffers para não relacionados
    virtual void setUnrelatedBuffers();
    //Reinicia a strategy
    void restartStrategy();

    int getTotalIndicacoesCompra() { return this.totalIndicacoesCompra; };
    int getTotalIndicacoesVenda() { return this.totalIndicacoesVenda; };
};

void NsVPRStg::set(double variacaoPercentil_, int atrasoVelas_){
    ZeroMemory(this.nsvprIndicador);
    this.nsvprIndicador = new NsVPR;
    nsvprIndicador.set(variacaoPercentil_, atrasoVelas_);
    this.setUltimoSinal(NONE_SIGN);
    this.restartStrategy();
}

void NsVPRStg::set(){
    this.set(0.05, 36);
}

void NsVPRStg::setBuffersRelation(){
    relatMxM = getInitialBufferRelation(nsvprIndicador.getBufferLast(NS_MAXM));
    relatMxS = getInitialBufferRelation(nsvprIndicador.getBufferLast(NS_MAXS));
    relatMxD = getInitialBufferRelation(nsvprIndicador.getBufferLast(NS_MAXD));
    relatMeio = getInitialBufferRelation(nsvprIndicador.getBufferLast(NS_MID));
    relatMiD = getInitialBufferRelation(nsvprIndicador.getBufferLast(NS_MIND));
    relatMiS = getInitialBufferRelation(nsvprIndicador.getBufferLast(NS_MINS));
    relatMiM = getInitialBufferRelation(nsvprIndicador.getBufferLast(NS_MINM));
}

ENUM_STRATEGY_SIGN NsVPRStg::getSign(){
    //Contadores de indicação
    int indicacoesCompra = 0;
    int indicacoesVenda = 0;
    relatMxM = getBufferRelation(nsvprIndicador.getBufferLast(NS_MAXM), relatMxM);
    relatMxS = getBufferRelation(nsvprIndicador.getBufferLast(NS_MAXS), relatMxS);
    relatMxD = getBufferRelation(nsvprIndicador.getBufferLast(NS_MAXD), relatMxD);
    relatMeio = getBufferRelation(nsvprIndicador.getBufferLast(NS_MID), relatMeio);
    relatMiD = getBufferRelation(nsvprIndicador.getBufferLast(NS_MIND), relatMiD);
    relatMiS = getBufferRelation(nsvprIndicador.getBufferLast(NS_MINS), relatMiS);
    relatMiM = getBufferRelation(nsvprIndicador.getBufferLast(NS_MINM), relatMiM);
    //Meio
    if(relatMeio == CROSSED_DOWN) //Cruzamento do meio para baixo é uma compra forte
        indicacoesCompra += 3;
    if(relatMeio == CROSSED_UP) //Cruzamento do meio para cima é uma venda forte
        indicacoesVenda += 3;
    //Maxs
    if(relatMxM == CROSSED_UP) //Cruzamento da máxima do mês para cima é uma compra forte
        indicacoesCompra += 3;
    if(relatMxM == CROSSED_DOWN) //Cruzamento da máxima do mês para baixo é uma venda forte
        indicacoesVenda += 3;
    if(relatMxS == CROSSED_UP) //Cruzamento da máxima da semana para cima é uma compra média
        indicacoesCompra += 2;
    if(relatMxS == CROSSED_DOWN) //Cruzamento da máxima da semana para baixo é uma venda média
        indicacoesVenda += 2;
    if(relatMxD == CROSSED_UP) //Cruzamento da máxima do dia para cima é uma compra fraca
        indicacoesCompra += 1;
    if(relatMxD == CROSSED_DOWN) //Cruzamento da máxima do dia para baixo é uma venda fraca
        indicacoesVenda += 1;
    //Mins
    if(relatMiM == CROSSED_UP) //Cruzamento da mínima do mês para cima é uma compra forte
        indicacoesCompra += 3;
    if(relatMiM == CROSSED_DOWN) //Cruzamento da mínima do mês para baixo é uma venda forte
        indicacoesVenda += 3;
    if(relatMiS == CROSSED_UP) //Cruzamento da mínima da semana para cima é uma compra média
        indicacoesCompra += 2;
    if(relatMiS == CROSSED_DOWN) //Cruzamento da mínima da semana para baixo é uma venda média
        indicacoesVenda += 2;
    if(relatMiD == CROSSED_UP) //Cruzamento da mínima do dia para cima é uma compra fraca
        indicacoesCompra += 1;
    if(relatMiD == CROSSED_DOWN) //Cruzamento da mínima do dia para baixo é uma venda fraca
        indicacoesVenda += 1;
    totalIndicacoesCompra += indicacoesCompra;
    totalIndicacoesVenda += indicacoesVenda;
    //Verificação das indicações e retorno
    if(indicacoesCompra > 0 || indicacoesVenda > 0){
        if(indicacoesCompra > indicacoesVenda){
            this.setUltimoSinal(this.getBuySignLevel(indicacoesCompra));
            return this.getUltimoSinal();
        }else{
            this.setUltimoSinal(this.getSellSignLevel(indicacoesVenda));
            return this.getUltimoSinal();
        }
    }else
        return NONE_SIGN;
}

void NsVPRStg::setUnrelatedBuffers(){
    relatMxM = UNRELATED_TO_BUFFER;
    relatMxS = UNRELATED_TO_BUFFER;
    relatMxD = UNRELATED_TO_BUFFER;
    relatMeio = UNRELATED_TO_BUFFER;
    relatMiD = UNRELATED_TO_BUFFER;
    relatMiS = UNRELATED_TO_BUFFER;
    relatMiM = UNRELATED_TO_BUFFER;
}

void NsVPRStg::restartStrategy(){
    this.setUnrelatedBuffers();
}